//Progetto Esame di Programmazione Procedurale 2021 - "TRANNOI" - Carlicchi Francesco (matricola 331792)
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "gamelib.h"


//VARIABILI GLOBALI

//Puntatore all'array dinamico che contiene i giocatori
static struct Giocatore* giocatori = NULL;
//Puntatore alla stanza in cui iniziano la partita tutti i giocatori
static struct Stanza* stanza_inizio = NULL;
//Puntatore alla prima stanza della lista stanze
static struct Stanza* lista_stanze = NULL;
//Numero di quest da completare per vincere
static unsigned short quest_da_finire = 0;
//Numero di quest completate dagli astronauti nella partita
static unsigned short quest_completate = 0;
//Numero di giocatori totali
static int numero_giocatori = 0;
//Token per il turno di un giocatore (fare un'azione consuma il token e si passa al giocatore successivo)
static bool token = false;
//Variabile booleana che gestisce se ci troviamo in partita o meno
static bool inGioco = false;


//DICHIARAZIONE DELLE FUNZIONI (per le info sulle funzioni, guardarne i commenti sulle rispettive definizioni)

//Funzioni di utilità
static void clear_buffer(void);
static void swap(int*, int*);
static int random(int, int);
static void random_senza_ripetizioni(int*, int);
//Funzioni per il gioco
static void titolo(void);
static void stampa_nome_giocatore(enum Nome_giocatore);
static void stampa_stato_giocatore(enum Stato_giocatore);
static void stampa_tipo_stanza(enum Tipo_stanza);
static void stampa_nome_stanza(enum Nome_stanza);
static void stampa_lista_giocatori(void);
static void menu_impostazioni(void);
static void inizia_gioco(void);
static void info_turno(struct Giocatore*);
static void crea_stanza(struct Stanza*, enum Punti_cardinali);
static struct Stanza* controlla_esistenza_stanza(short int, short int);
static void dealloca(void);
static int conta_giocatori(struct Stanza*, enum Stato_giocatore);
static char simbolo_mappa(enum Tipo_stanza);
static void stampa_mappa(struct Stanza*);
static void stampa_lista_botole(void);
static void esci_partita(void);
static void controllo_vittoria(void);
static bool quest_fili(void);
static bool quest_operazioni(int);
static bool quest_dado(void);
static bool quest_hack(void);
static bool quest_carte(void);
//Funzioni relative alle azioni da fare in gioco
static void avanza(struct Giocatore*);
static void esegui_quest(struct Stanza*);
static void chiamata_emergenza(struct Stanza*);
static void uccidi_astronauta(struct Giocatore*);
static void usa_botola(struct Giocatore*);
static void sabotaggio(struct Stanza*);


//DEFINIZIONE DELLE FUNZIONI STATICHE

//Funzione per ripulire il buffer dopo un inserimento da tastiera
static void clear_buffer(void){
  while((getchar()) != '\n');
}

//Funzione per invertire il contenuto di due variabili
static void swap(int* a, int* b){
  if(a != b && a != NULL && b != NULL){
    int temp = *a;
    *a = *b;
    *b = temp;
  }
}

//Generatore di numeri casuali in un intervallo [min, max]
static int random(int min, int max){
  if(min > max)
   swap(&min, &max);
  return rand() % (max - min + 1) + min;
}

//Funzione per randomizzare un array dato
static void random_senza_ripetizioni(int* array, int numero_elementi){
  if(array != NULL)
    for(int i = numero_elementi - 1; i > 0; i--)
      swap(&array[random(0, i)], &array[i]);
}

//Funzione per la stampa del titolo del gioco
static void titolo(void){
  system("clear");
  printf("\n\t" blue_color " _______  _______  _______    __    _  " yellow_color "__      __ _______  __ \n");
  printf("\t" blue_color "|_-_-_-_||-  _  -\\/ " yellow_color "x _ x" blue_color " \\_ |  \\  | |" yellow_color "|  \\    / //  ___  \\|--|" blue_color "      	 ......\n");
  printf("\t" blue_color "   | |   |  |_|  /|  " yellow_color "/_\\" blue_color "  | ||   \\ | |" yellow_color "|   \\  / / | |   | | __"blue_color "          _/_*_*_0._      " white_color "*\n");
  printf("\t" blue_color "   | |   |  _  _/ |  ___  | || |\\ \\| |" yellow_color "| |\\ |/ /  | |   | ||  |  " white_color "* * *" blue_color " (__o__o__o_)<" yellow_color "=" white_color "         .\n");
  printf("\t" blue_color "   \\ |   \\ | \\ \\  | |   | |_|| | \\   /" yellow_color "| | \\_  |  \\ |___| /|  |     " white_color "                       o     " blue_color "||\n");
  printf("\t    \\|    \\|  \\_| |/    |_/  |_|  \\_/ " yellow_color "|_|   |_|   \\_____/ \\__/" white_color "    *   o      .      *           " blue_color "/\\\n");
  printf("\t                                                                                               /" yellow_color "**" blue_color "\\\n");
  printf("    ____/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___/\\___|  |\n");
  printf("   /   ____|____|____|____|____|____|____|____|____|____|____|____|____|____|____|____|____|____|____|____/\n");
  printf("   |  |                                                                             \n   \\__/\n" white_color);
}

//Funzione per stampare la stringa nome del giocatore
static void stampa_nome_giocatore(enum Nome_giocatore nome){
  switch(nome){
    case 0:
      printf("Rosso");
      break;
    case 1:
      printf("Blu");
      break;
    case 2:
      printf("Giallo");
      break;
    case 3:
      printf("Verde");
      break;
    case 4:
      printf("Rosa");
      break;
    case 5:
      printf("Nero");
      break;
    case 6:
      printf("Arancio");
      break;
    case 7:
      printf("Marrone");
      break;
    case 8:
      printf("Bianco");
      break;
    case 9:
      printf("Viola");
  }
}

//Funzione per stampare la stringa stato del giocatore
static void stampa_stato_giocatore(enum Stato_giocatore stato){
  switch(stato){
    case 0:
      printf("astronauta");
      break;
    case 1:
      printf("impostore");
      break;
    case 2:
      printf("assassinato");
      break;
    case 3:
      printf("defenestrato");
  }
}

//Funzione per stampare la stringa tipo della stanza
static void stampa_tipo_stanza(enum Tipo_stanza stanza){
  switch(stanza){
    case 0:
      printf("vuota");
      break;
    case 1:
      printf("quest semplice");
      break;
    case 2:
      printf("quest complicata");
      break;
    case 3:
      printf("botola");
  }
}

//Funzione per stampare la stringa nome della stanza
static void stampa_nome_stanza(enum Nome_stanza nome){
  switch(nome){
    case 0:
      printf("caffetteria");
      break;
    case 1:
      printf("elettrica");
      break;
    case 2:
      printf("generatore");
      break;
    case 3:
      printf("motore");
      break;
    case 4:
      printf("reattore");
      break;
    case 5:
      printf("navigazione");
      break;
    case 6:
      printf("sicurezza");
      break;
    case 7:
      printf("magazzino");
      break;
    case 8:
      printf("armeria");
      break;
    case 9:
      printf("aereazione");
  }
}

//Funzione per stampare l'elenco di tutti i giocatori (nome + stato + posizione)
static void stampa_lista_giocatori(void){
  printf("\n\n\t" yellow_color " +--------------+---------------+-------+\n");
  printf("\t | GIOCATORE    | STATO         | X-Y   |\n");
  printf("\t +--------------+---------------+-------+\n");
  if(giocatori != NULL && numero_giocatori > 0){
    for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
      printf("\t | ");
      stampa_nome_giocatore(i -> nome);
      printf("  \t| ");
      stampa_stato_giocatore(i -> stato);
      printf("\t| %hu-%hu\t|\n", i -> posizione -> x_pos, i -> posizione -> y_pos);
    }
  }else{
    printf("\t | ----         | ----          | ---   |\n");
  }
  printf("\t +--------------+---------------+-------+\n");
}

//Funzione per la stampa del menu delle impostazioni di gioco
static void menu_impostazioni(void){
  titolo();
  printf("\n\n\t" yellow_color " IMPOSTA LA TUA PARTITA\n\t 1] CREA I GIOCATORI\n\t 2] IMPOSTA IL NUMERO DI QUEST DA COMPLETARE\n\t 3] LISTA DEI GIOCATORI\n\t 4] TORNA AL MENU PRINCIPALE\n\t  > " blue_color);
}

//Funzione per salvare le impostazioni e iniziare a giocare
static void inizia_gioco(void){
  printf("\n\n\t" yellow_color " Salvataggio delle impostazioni in corso...\n");
  sleep(2);
}

//Funzione per stampare le info relative al giocatore e allo stato della partita ad ogni turno
static void info_turno(struct Giocatore* corrente){
  if(corrente != NULL){
    system("clear");
    bool trovato = false;
    printf("\n\n\t" yellow_color "NOME: " white_color);
    stampa_nome_giocatore(corrente -> nome);
    printf("\n\t" yellow_color "RUOLO: ");
    if(corrente -> stato == astronauta)
      printf(blue_color);
    else
      printf(red_color);
    stampa_stato_giocatore(corrente -> stato);
    printf("\n\t" yellow_color "STANZA ATTUALE: " white_color);
    stampa_nome_stanza(corrente -> posizione -> nome_stanza);
    printf("\n\t" yellow_color "TIPO STANZA: " white_color);
    stampa_tipo_stanza(corrente -> posizione -> tipo);
    printf("\n\t" yellow_color "QUEST COMPLETATE: " green_color "%hu/%hu", quest_completate, quest_da_finire);
    printf("\n\t" yellow_color "MAPPA:\n" white_color);
    stampa_mappa(corrente -> posizione);
    printf("\n\t" yellow_color "GIOCATORI PRESENTI:" white_color);
    for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
      if(i != corrente && i -> posizione == corrente -> posizione && i -> stato != assassinato){
        printf("\n\t > ");
        stampa_nome_giocatore(i -> nome);
        trovato = true;
      }
    }
    if(!trovato)
      printf("\n\t ----");
    if(corrente -> stato == impostore){
      printf("\n\t" red_color "IMPOSTORI:" white_color);
      for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
        if(i -> stato == impostore){
          printf("\n\t > ");
          stampa_nome_giocatore(i -> nome);
        }
      }
    }
  }else{
    printf("\t" red_color "Le info non sono disponibili.\n\t" white_color);
  }
}

//Funzione per la creazione di una nuova stanza
static void crea_stanza(struct Stanza* precedente, enum Punti_cardinali direzione){
  if(precedente != NULL){
    struct Stanza* nuova_stanza = (struct Stanza*) malloc(sizeof(struct Stanza));
    if(nuova_stanza == NULL){
      printf("\t" red_color "Memoria esaurita.\n\t" white_color);
      exit(-1);
    }
    //Impostazione delle coordinate della nuova stanza in base a quella precedente e alla direzione in cui è stata creata
    switch (direzione) {
      case 0:
        nuova_stanza -> x_pos = precedente -> x_pos;
        nuova_stanza -> y_pos = (precedente -> y_pos) + 1;
        break;
      case 1:
        nuova_stanza -> x_pos = precedente -> x_pos;
        nuova_stanza -> y_pos = (precedente -> y_pos) - 1;
        break;
      case 2:
        nuova_stanza -> x_pos = (precedente -> x_pos) + 1;
        nuova_stanza -> y_pos = precedente -> y_pos;
        break;
      case 3:
        nuova_stanza -> x_pos = (precedente -> x_pos) - 1;
        nuova_stanza -> y_pos = precedente -> y_pos;
    }
    //Collegamento della nuova stanza con quelle adiacenti (se esistono) e viceversa (per tutti e quattro i lati della stanza)
    nuova_stanza -> nord = controlla_esistenza_stanza(nuova_stanza -> x_pos, (nuova_stanza -> y_pos) + 1);
    if(nuova_stanza -> nord != NULL)
      nuova_stanza -> nord -> sud = nuova_stanza;
    nuova_stanza -> sud = controlla_esistenza_stanza(nuova_stanza -> x_pos, (nuova_stanza -> y_pos) - 1);
    if(nuova_stanza -> sud != NULL)
      nuova_stanza -> sud -> nord = nuova_stanza;
    nuova_stanza -> est = controlla_esistenza_stanza((nuova_stanza -> x_pos) + 1, nuova_stanza -> y_pos);
    if(nuova_stanza -> est != NULL)
      nuova_stanza -> est -> ovest = nuova_stanza;
    nuova_stanza -> ovest = controlla_esistenza_stanza((nuova_stanza -> x_pos) - 1, nuova_stanza -> y_pos);
    if(nuova_stanza -> ovest != NULL)
      nuova_stanza -> ovest -> est = nuova_stanza;
    //Impostazione randomica del tipo della nuova stanza
    int tipo_nuova_stanza = random(0, 99);
    if(tipo_nuova_stanza < 25)
      nuova_stanza -> tipo = botola;
    else if(tipo_nuova_stanza < 40)
      nuova_stanza -> tipo = quest_complicata;
    else if(tipo_nuova_stanza < 70)
      nuova_stanza -> tipo = quest_semplice;
    else
      nuova_stanza -> tipo = vuota;
    //Impostazione dei campi rimanenti (puntatore, nome stanza e flag booleano per la chiamata dell'emergenza)
    nuova_stanza -> successiva = NULL;
    nuova_stanza -> nome_stanza = random(1, 9);
    nuova_stanza -> emergenza_chiamata = false;
    //Aggiunta in coda alla lista della nuova stanza
    struct Stanza* scorrimento = lista_stanze;
    while(scorrimento -> successiva != NULL){
      scorrimento = scorrimento -> successiva;
    }
    scorrimento -> successiva = nuova_stanza;
  }else{
    printf("\t" red_color "[ERRORE]: Impossibile creare nuova stanza.\n\t" white_color);
    exit(-1);
  }
}

//Funzione per controllare l'esistenza di una stanza (scorre la lista delle stanze e ritorna il puntatore alla stanza con quelle coordinate. Se non esiste ritorna NULL)
static struct Stanza* controlla_esistenza_stanza(short int x, short int y){
  struct Stanza* scorrimento_lista = lista_stanze;
  while(scorrimento_lista != NULL){
    if(scorrimento_lista -> x_pos == x && scorrimento_lista -> y_pos == y)
      return scorrimento_lista;
    scorrimento_lista = scorrimento_lista -> successiva;
  }
  return NULL;
}

//Funzione per deallocare la lista giocatori e le stanze
static void dealloca(void){
  if(giocatori != NULL){
    free(giocatori);
    giocatori = NULL;
  }
  struct Stanza* scorrimento_lista;
  while(lista_stanze != NULL){
    scorrimento_lista = lista_stanze;
    lista_stanze = lista_stanze -> successiva;
    free(scorrimento_lista);
  }
  stanza_inizio = NULL;
}

//Funzione per contare il numero di giocatori (se la stanza in input non esiste (corrente == NULL), la funzione ritorna il numero totale di giocatori con quello stato, altrimenti solo quelli relativi alla stanza puntata)
static int conta_giocatori(struct Stanza* corrente, enum Stato_giocatore stato){
  int contatore = 0;
  if(corrente == NULL){
    for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
      if(i -> stato == stato)
        contatore++;
    }
  }else{
    for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
      if(i -> posizione == corrente && i -> stato == stato)
        contatore++;
    }
  }
  return contatore;
}

//Funzione per stampare il simbolo del tipo della stanza sulla mappa
static char simbolo_mappa(enum Tipo_stanza tipo){
  switch(tipo){
    case 0:
      return '+';
    case 1:
      return 'q';
    case 2:
      return 'Q';
    case 3:
      return '#';
  }
  return ' ';
}

//Funzione per stampare la mappa di gioco (matrice 5x5 con al centro la stanza attuale del giocatore)
static void stampa_mappa(struct Stanza* centrale){
  if(centrale != NULL){
    int giocatori_locali;
    struct Stanza* corrente;
    for(int riga = centrale -> y_pos + 2; riga >= centrale -> y_pos - 2; riga--){
      printf("\t+-----+-----+-----+-----+-----+\n\t");
      //Stampa della prima parte di ciascuna casella della riga n-esima della mappa (con il numero di giocatori presenti in alto a sinistra)
      for(int colonna = centrale -> x_pos - 2; colonna <= centrale -> x_pos + 2; colonna++){
        corrente = controlla_esistenza_stanza(colonna, riga);
        if(corrente == NULL){
          printf("|     ");
        }else{
          giocatori_locali = conta_giocatori(corrente, astronauta) + conta_giocatori(corrente, impostore);
          if(giocatori_locali == 10)
            printf("|%d   ", giocatori_locali);
          else
            printf("|%d    ", giocatori_locali);
        }
      }
      printf("|\n\t");
      //Stampa della seconda parte di ciascuna casella della riga n-esima della mappa (con il tipo della stanza e l'evenuale presenza di cadaveri)
      for(int colonna = centrale -> x_pos - 2; colonna <= centrale -> x_pos + 2; colonna++){
        corrente = controlla_esistenza_stanza(colonna, riga);
        if(corrente == NULL){
          printf("|  ?  ");
        }else{
          putchar('|');
          if(corrente == centrale && conta_giocatori(corrente, assassinato) > 0)
            printf(red_color);
          printf("  %c  " white_color, simbolo_mappa(corrente -> tipo));
        }
      }
      printf("|\n\t|     |     |     |     |     |\n");
    }
    printf("\t+-----+-----+-----+-----+-----+");
  }else{
      printf("\t" red_color "La mappa non e' disponibile." white_color);
  }
}

//Funzione per stampare la lista di stanze con una botola presente
static void stampa_lista_botole(void){
  struct Stanza* scorrimento_lista = lista_stanze;
  int indice = 0;
  printf("\n\n\tStanze con botola:");
  printf("\n\t+-------+---------------+-------+-------+----------------+");
  printf("\n\t| N     | Nome          |  X    |  Y    | Astronauti     |");
  printf("\n\t+-------+---------------+-------+-------+----------------+");
  while(scorrimento_lista != NULL){
    if(scorrimento_lista -> tipo == botola){
      printf("\n\t| %d\t| ", ++indice);
      stampa_nome_stanza(scorrimento_lista -> nome_stanza);
      printf("\t| ");
      if(scorrimento_lista -> x_pos >= 0)
        putchar(' ');
      printf("%hd\t| ", scorrimento_lista -> x_pos);
      if(scorrimento_lista -> y_pos >= 0)
        putchar(' ');
      printf("%hd\t| %d\t         |", scorrimento_lista -> y_pos, conta_giocatori(scorrimento_lista, astronauta));
    }
    scorrimento_lista = scorrimento_lista -> successiva;
  }
  printf("\n\t+-------+---------------+-------+-------+----------------+\n");
}

//Funzione per uscire dalla partita e tornare al menu principale
static void esci_partita(void){
  system("clear");
  char scelta;
  do{
    system("clear");
    printf("\n\tSei sicuro di voler uscire? I progressi andranno persi. Inserisci s/S oppure n/N.\n\t > ");
    scelta = getchar();
    clear_buffer();
    switch(scelta){
      case 's':
      case 'S':
        break;
      case 'n':
      case 'N':
        return;
      default:
        printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: s-S-n-N.\n" white_color);
        sleep(2);
    }
  }while(scelta != 's' && scelta != 'S');
  inGioco = false;
  token = false;
}

//Funzione per controllare la vittoria di una squadra (impostori o astronauti) dopo ogni turno
static void controllo_vittoria(void){
  system("clear");
  if(conta_giocatori(NULL, impostore) < 1 || quest_completate >= quest_da_finire){
    printf("\n\n\t" blue_color "GLI ASTRONAUTI HANNO VINTO!!\n");
    sleep(3);
    inGioco = false;
  }else if(conta_giocatori(NULL, astronauta) < 1){
    printf("\n\n\t" red_color "GLI IMPOSTORI HANNO VINTO!!\n");
    sleep(3);
    inGioco = false;
  }
}

//Funzione per il minigioco dei fili (quest semplice). Si devono collegare i fili di dx con quelli corrispondenti di sx.
static bool quest_fili(void){
  system("clear");
  printf("\n\n\t" blue_color "-> FIX-CABLES.exe ...\n\n" white_color);
  printf("\t +-----+-----------+-----+\n");
  printf("\t |     " yellow_color "|           |" white_color "     |\n");
  //Generazione randomica delle quattro lettere relative ai fili di sx
  int porteA[4] = {random(65,70), random(71,76), random(77,83), random(84,90)};
  int porteB[4];
  //Randomizzazione delle lettere dei fili di dx in base a quelle generate a sx
  for(int i = 0; i < 4; i++)
    porteB[i] = porteA[i];
  random_senza_ripetizioni(porteB, 4);
  //stampa dei fili a schermo
  for(int i = 0; i < 4; i++){
    printf("\t%d| %c" yellow_color "-" red_color "==" yellow_color "0" red_color "==" yellow_color ">     <" red_color "==" yellow_color "0" red_color "==" yellow_color "-" white_color "%c |\n", i + 1, (char) porteA[i], (char) porteB[i]);
  }
  printf("\t |     " yellow_color "|           |" white_color "     |\n");
  printf("\t +-----+-----------+-----+\n");
  //Generazione della stringa corretta da confrontare con quella inserita
  unsigned int sequenza = 10203040, risposta;
  for(int a = 0; a < 4; a++){
    for(int b = 0; b < 4; b++){
      if(porteA[a] == porteB[b]){
        sequenza += (b + 1) * pow(10, 2 * (3 - a));
        break;
      }
    }
  }
  printf("\tInserisci una stringa numerica che collega in ordine crescente ciascun filo di sx con ciascun filo di dx. (ES: 13213442)\n\tSEQUENZA > ");
  scanf("%u", &risposta);
  clear_buffer();
  return (risposta == sequenza ? true : false);
}

//Funzione per il minigioco delle operazioni (quest semplice e complicata). Si deve risolvere un'operazione (+, -, *).
static bool quest_operazioni(int difficolta){
  system("clear");
  printf("\n\n\t" blue_color "-> MATH.exe ...\n\n" white_color);
  //Generazione randomica degli operandi
  int a  = random(1, 5 * pow(2, difficolta));
  int b = random(1, 5 * pow(2, difficolta));
  int risposta;
  printf("\tCalcola la segunte operazione: ");
  switch(random(0, 2)){
    case 0:
      printf ("\t%d + %d = ", a, b);
      scanf("%d", &risposta);
      clear_buffer();
      return (risposta == a + b ? true : false);
    case 1:
      printf ("\t%d - %d = ", a, b);
      scanf("%d", &risposta);
      clear_buffer();
      return (risposta == a - b ? true : false);
    case 2:
      printf ("\t%d * %d = ", a, b);
      scanf("%d", &risposta);
      clear_buffer();
      return (risposta == a * b ? true : false);
  }
  return false;
}

//Funzione per il minigioco dei dadi (quest semplice). Si fa una puntata sul lancio di un dado.
static bool quest_dado(void){
  int scelta, dado = random(1, 6);
  do{
    system("clear");
    printf("\n\n\t" blue_color "-> DICE.exe ...\n\n" white_color);
    printf("\tEffettua una puntata tra le seguenti.\n\t1] Numeri PARI\n\t2] Numeri DISPARI\n\t3] 1-2-3\n\t4] 4-5-6\n\t > ");
    scanf("%d", &scelta);
    clear_buffer();
    if(scelta < 1 || scelta > 4){
      printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: 1-2-3-4.\n" white_color);
      sleep(2);
    }
  }while(scelta < 1 || scelta > 4);
  printf("\tDADO: %d\n", dado);
  sleep(2);
  switch(scelta){
    case 1:
      return (dado % 2 == 0 ? true : false);
    case 2:
      return (dado % 2 != 0 ? true : false);
    case 3:
      return (dado < 4 ? true : false);
    case 4:
      return (dado > 3 ? true : false);
  }
  return false;
}

//Funzione per il minigioco dell'hack (quest complicata). Si deve trovare una stringa di numeri all'interno di una matrice generata a random.
static bool quest_hack(void){
  system("clear");
  printf("\n\n\t" blue_color "-> TERMINAL-HACK.exe ...\n\n" white_color);
  int codice = random(0, 99), riga, colonna;
  int numeri[100];
  //Generazione della matrice
  for(int i = 0; i < 100; i++)
    numeri[i] = random(10, 99);
  printf("\t     00   01   02   03   04   05   06   07   08   09");
  //Stampa della matrice
  for(int r = 0; r < 10; r++){
    printf("\n\t   +----+----+----+----+----+----+----+----+----+----+\n\t0%d ", r);
    for(int c = 0; c < 10; c++)
      printf("| %d ", numeri[c + 10 * r]);
    putchar('|');
  }
  printf("\n\t   +----+----+----+----+----+----+----+----+----+----+\n");
  printf("\n\tCODICE: %d %d %d %d\n\tInserisci riga e colonna del primo elemento del codice nella matrice.\n\tRIGA > ", numeri[codice], numeri[(codice + 1) % 100], numeri[(codice + 2) % 100], numeri[(codice + 3) % 100]);
  scanf("%d", &riga);
  clear_buffer();
  printf("\tCOLONNA > ");
  scanf("%d", &colonna);
  clear_buffer();
  return (colonna + 10 * riga == codice ? true : false);
}

//Funzione per il minigioco delle carte (quest complicata). Si può scegliere di pescare delle carte, con l'obbiettivo di avere una somma maggiore a quella del banco, ma che non ecceda 21.
static bool quest_carte(void){
  int carte_giocatore = random(2, 20);
  int carte_banco = 0;
  char scelta;
  do{
    //Turno del giocatore
    do{
      system("clear");
      printf("\n\n\t" blue_color "-> ASTRO-JACK.exe ...\n\n" white_color);
      printf("\tPer vincere devi avere una somma di carte superiore al banco, che non ecceda 21.\n\tIl banco peschera' una carta finche' non avra' una somma maggiore di 17.\n");
      printf("\tTOTALE CARTE: %d\n\tBANCO: **\n\n\tVuoi pescare un'altra carta? Inserisci s/S oppure n/N.\n\t> ", carte_giocatore);
      scelta = getchar();
      clear_buffer();
      if(scelta != 's' && scelta != 'S' && scelta != 'n' && scelta != 'N'){
        printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: s-S-n-N.\n" white_color);
        sleep(2);
      }
    }while(scelta != 's' && scelta != 'S' && scelta != 'n' && scelta != 'N');
    //Un'altra carta viene pescata e aggiunta al totale
    if(scelta == 's' || scelta == 'S')
        carte_giocatore += random(1, 10);
  }while(scelta != 'n' && scelta != 'N' && carte_giocatore < 21);
  //Turno del banco (automatico)
  while(carte_banco < 17)
    carte_banco += random(1, 10);
  system("clear");
  printf("\n\n\t" blue_color "-> ASTRO-JACK.exe ...\n\n" white_color);
  printf("\tTOTALE CARTE: %d\n\tBANCO: %d\n", carte_giocatore, carte_banco);
  if((carte_giocatore > carte_banco || carte_banco > 21) && carte_giocatore < 22){
    printf("\tIl giocatore ha vinto!\n");
    sleep(2);
    return true;
  }else{
    printf("\tIl banco ha vinto!\n");
    sleep(2);
    return false;
  }
}

//Funzione per avanzare in una nuova stanza (astronauti + impostori)
static void avanza(struct Giocatore* corrente){
  if(corrente != NULL){
    if(corrente -> posizione != NULL){
      int scelta;
      do{
        info_turno(corrente);
        printf("\n\n\t" yellow_color "AVANZA" white_color "\n\tIn che direzione vuoi avanzare?\n\t1] Nord\n\t2] Sud\n\t3] Est\n\t4] Ovest\n\t5] Rimani fermo\n\t> ");
        scanf("%d", &scelta);
        clear_buffer();
        switch(scelta) {
          case 1:
            if(corrente -> posizione -> nord == NULL) //Se la stanza non esiste viene creata
              crea_stanza(corrente -> posizione, nord);
            corrente -> posizione = corrente -> posizione -> nord;  //Il giocatore viene spostato
            printf("\t" green_color "Ti sei spostato nella stanza a nord!\n\t" white_color);
            sleep(2);
            break;
          case 2:
            if(corrente -> posizione -> sud == NULL)
              crea_stanza(corrente -> posizione, sud);
            corrente -> posizione = corrente -> posizione -> sud;
            printf("\t" green_color "Ti sei spostato nella stanza a sud!\n\t" white_color);
            sleep(2);
            break;
          case 3:
            if(corrente -> posizione -> est == NULL)
              crea_stanza(corrente -> posizione, est);
            corrente -> posizione = corrente -> posizione -> est;
            printf("\t" green_color "Ti sei spostato nella stanza a est!\n\t" white_color);
            sleep(2);
            break;
          case 4:
            if(corrente -> posizione -> ovest == NULL)
              crea_stanza(corrente -> posizione, ovest);
            corrente -> posizione = corrente -> posizione -> ovest;
            printf("\t" green_color "Ti sei spostato nella stanza a ovest!\n\t" white_color);
            sleep(2);
            break;
          case 5:
            printf("\t" green_color "Sei rimasto nella stessa stanza!\n\t" white_color);
            sleep(2);
            break;
          default:
            printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: 1-2-3-4-5.\n\t" white_color);
            sleep(1);
        }
      }while(scelta < 1 || scelta > 5);
      token = false;
    }else{
      printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n\t" white_color);
      sleep(1);
    }
  }else{
    printf("\t" red_color "[ERRORE]: Il giocatore non e' valido.\n\t" white_color);
    sleep(1);
  }
}

//Funzione per eseguire una quest (astronauti)
static void esegui_quest(struct Stanza* corrente){
  if(corrente != NULL){
    bool completata;
    if(corrente -> tipo == quest_semplice){
      //Scelta tra 3 minigiochi semplici
      switch(random(0, 2)){
        case 0:
          completata = quest_fili();
          break;
        case 1:
          completata = quest_operazioni(random(1, 3));
          break;
        case 2:
          completata = quest_dado();
      }
      if(completata){
        quest_completate++;
        corrente -> tipo = vuota;
        printf("\t" green_color "QUEST semplice eseguita!\n" white_color);
      }else{
        printf("\t" red_color "QUEST semplice fallita!\n" white_color);
      }
      sleep(2);
      token = false;
    }else if(corrente -> tipo == quest_complicata){
      //Scelta tra 3 minigiochi complicati
      switch(random(0, 2)){
        case 0:
          completata = quest_hack();
          break;
        case 1:
          completata = quest_operazioni(random(4, 5));
          break;
        case 2:
          completata = quest_carte();
      }
      if(completata){
        quest_completate += 2;
        corrente -> tipo = vuota;
        printf("\t" green_color "QUEST complicata eseguita!\n" white_color);
      }else{
        printf("\t" red_color "QUEST complicata fallita!\n" white_color);
      }
      sleep(2);
      token = false;
    }else{
      printf("\t" green_color "Non ci sono QUEST da fare in questa stanza!\n" white_color);
      sleep(2);
    }
  }else{
    printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n" white_color);
    sleep(1);
  }
}

//Funzione per richiamare un'emergenza in caso di cadaveri (astronauti + impostori)
static void chiamata_emergenza(struct Stanza* scena_crimine){
  if(scena_crimine != NULL){
    if(scena_crimine -> emergenza_chiamata == false){
      //Ciclo per ricercare la presenza di cadaveri nella stanza
      for(struct Giocatore* cadavere = giocatori; cadavere < giocatori + numero_giocatori; cadavere++){
        if(cadavere -> posizione == scena_crimine && cadavere -> stato == assassinato){
          int probabilita;
          int n_impostori = conta_giocatori(scena_crimine, impostore);
          int n_astronauti = conta_giocatori(scena_crimine, astronauta);
          scena_crimine -> emergenza_chiamata = true;
          token = false;
          //Rimozione dei cadaveri nella stanza
          for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
            if(i -> stato == assassinato && i -> posizione == scena_crimine)
                i -> posizione = NULL;
          }
          //Ricerca di eventuali giocatori da defenestrare
          for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
            if(i -> posizione == scena_crimine){
              //Calcolo della probabilitá di defenestrazione
              if(i -> stato == impostore)
                probabilita = 30 + 20 * n_astronauti - 30 * (n_impostori - 1);
              else
                probabilita = 30 + 20 * n_impostori - 30 * (n_astronauti - 1);
              //Tentativo di defenestrazione
              if(probabilita > 0 && random(0, 99) < probabilita){
                  i -> stato = defenestrato;
                  i -> posizione = NULL;
                  printf("\t" green_color);
                  stampa_nome_giocatore(i -> nome);
                  printf(" e' stato defenestrato!\n" white_color);
                  sleep(2);
                  return;
              }
            }
          }
          printf("\t" green_color "Nessuno e' stato defenestrato!\n" white_color);
          sleep(2);
          return;
        }
      }
      printf("\t" green_color "Non ci sono cadaveri nella stanza!\n" white_color);
      sleep(2);
    }else{
      printf("\t" green_color "L'emergenza e' gia stata chiamata una volta in questa stanza!\n" white_color);
      sleep(2);
    }
  }else{
    printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n" white_color);
    sleep(1);
  }
}

//Funzione per uccidere un astronauta (impostori)
static void uccidi_astronauta(struct Giocatore* killer){
  if(killer != NULL){
    struct Stanza* corrente = killer -> posizione;
    if(corrente != NULL){
      //Ciclo per verificare se ci sono astronauti da uccidere
      for(struct Giocatore* i = giocatori; i < giocatori + numero_giocatori; i++){
        if(i -> posizione == corrente && i -> stato == astronauta){
          char nome_vittima[10];
          enum Nome_giocatore vittima;
          //Richiesta al giocatore di chi si vuole uccidere
          do{
            info_turno(killer);
            printf("\n\n\tChi vuoi uccidere? Scrivi 'annulla' per tornare indietro.\n\t> ");
            fgets(nome_vittima, 10, stdin);
            //Rimozione di caratteri indesiderati e pulizia del buffer
            if(nome_vittima[strlen(nome_vittima) - 1] == '\n')
              nome_vittima[strlen(nome_vittima) - 1] = '\0';
            else
              clear_buffer();
            //Verifica della validità del nome inserito e conversione del nome da string a enum Nome_giocatore
            if(!strcmp(nome_vittima, "Rosso") || !strcmp(nome_vittima, "rosso")){
              vittima = Rosso;
            }else if(!strcmp(nome_vittima, "Blu") || !strcmp(nome_vittima, "blu")){
              vittima = Blu;
            }else if(!strcmp(nome_vittima, "Giallo") || !strcmp(nome_vittima, "giallo")){
              vittima = Giallo;
            }else if(!strcmp(nome_vittima, "Verde") || !strcmp(nome_vittima, "verde")){
              vittima = Verde;
            }else if(!strcmp(nome_vittima, "Rosa") || !strcmp(nome_vittima, "rosa")){
              vittima = Rosa;
            }else if(!strcmp(nome_vittima, "Nero") || !strcmp(nome_vittima, "nero")){
              vittima = Nero;
            }else if(!strcmp(nome_vittima, "Arancio") || !strcmp(nome_vittima, "arancio")){
              vittima = Arancio;
            }else if(!strcmp(nome_vittima, "Marrone") || !strcmp(nome_vittima, "marrone")){
              vittima = Marrone;
            }else if(!strcmp(nome_vittima, "Bianco") || !strcmp(nome_vittima, "bianco")){
              vittima = Bianco;
            }else if(!strcmp(nome_vittima, "Viola") || !strcmp(nome_vittima, "viola")){
              vittima = Viola;
            }else{
              if(strcmp(nome_vittima, "annulla")){
                printf("\t" red_color "Non esiste un astronauta con questo nome!\n" white_color);
                sleep(2);
              }
              continue;
            }
            //Verifica dell'esistenza dell'astronauta con il nome inserito nella stanza corrente
            for(struct Giocatore* j = giocatori; j < giocatori + numero_giocatori; j++){
              if(j -> posizione == corrente && j -> nome == vittima && j -> stato == astronauta){
                token = false;
                j -> stato = assassinato;
                printf("\t" red_color);
                stampa_nome_giocatore(j -> nome);
                printf(" e' stato ucciso.\n" white_color);
                sleep(2);
                //Calcolo della probabilità di essere stato scoperto da altri astronauti
                int probabilita = 0;
                for(struct Giocatore* k = giocatori; k < giocatori + numero_giocatori; k++){
                  if(k -> stato == astronauta){
                    if(k -> posizione == corrente)
                      probabilita += 50;
                    else if(k -> posizione == corrente -> nord || k -> posizione == corrente -> sud || k -> posizione == corrente -> est || k -> posizione == corrente -> ovest)
                      probabilita += 5;
                  }
                }
                //Tentativo di defenestrazione
                if(random(0, 99) < probabilita){
                    killer -> stato = defenestrato;
                    killer -> posizione = NULL;
                    j -> posizione = NULL;
                    printf("\t" red_color "Sei stato scoperto! Vieni quindi defenestrato!\n" white_color);
                    sleep(2);
                }
                return;
              }
            }
            printf("\t" red_color "Non esiste un astronauta vivo con questo nome nella stanza!\n" white_color);
            sleep(2);
          }while(strcmp(nome_vittima, "annulla"));
          return;
        }
      }
      printf("\t" green_color "Non ci sono astronauti da uccidere!\n" white_color);
      sleep(2);
    }else{
      printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n" white_color);
      sleep(1);
    }
  }else{
    printf("\t" red_color "[ERRORE]: Il giocatore non e' valido.\n" white_color);
    sleep(1);
  }
}

//Funzione per utilizzare la botola (impostori)
static void usa_botola(struct Giocatore* attuale){
  if(attuale != NULL){
    struct Stanza* corrente = attuale -> posizione;
    if(corrente != NULL){
      if(corrente -> tipo == botola){
        //Ricerca e conteggio delle stanze con la botola diverse da quella attuale
        struct Stanza* scorrimento_lista = lista_stanze;
        int n_stanze_botola = 0;
        while(scorrimento_lista != NULL){
          if(scorrimento_lista -> tipo == botola && scorrimento_lista != corrente)
            n_stanze_botola++;
          scorrimento_lista = scorrimento_lista -> successiva;
        }
        //Verifica se esistono altre stanze con la botola
        if(n_stanze_botola > 0){
          short int x, y;
          struct Stanza* stanza_arrivo;
          //Richiesta della stanza in cui dirigersi e verifica della sua validità
          do{
            info_turno(attuale);
            stampa_lista_botole();
            printf("\tInserisci le coordinate della stanza in cui vuoi andare (x-y):\n\tX > ");
            scanf("%hu", &x);
            clear_buffer();
            printf("\tY > ");
            scanf("%hu", &y);
            clear_buffer();
            stanza_arrivo = controlla_esistenza_stanza(x, y);
            if(stanza_arrivo == NULL){
              printf("\t" red_color "[ERRORE]: Stanza inesistente. Inserire le coordinate di una stanza sopra elencata.\n" white_color);
              sleep(2);
            }else if(stanza_arrivo -> tipo != botola){
              printf("\t" red_color "[ERRORE]: Stanza non valida. Inserire le coordinate di una stanza sopra elencata.\n" white_color);
              sleep(2);
            }else if(stanza_arrivo == corrente){
              printf("\t" red_color "Ti trovi gia in quella stanza.\n" white_color);
              sleep(2);
            }else{
              break;
            }
          }while(1);
          token = false;
          attuale -> posizione = stanza_arrivo;
          printf("\t" green_color "Ti sei spostato nella nuova stanza!\n" white_color);
          sleep(2);
          //Verifica della possibilità di fare un'imboscata agli astronauti
          if(conta_giocatori(stanza_arrivo, astronauta) > 0){
            char scelta;
            do{
              info_turno(attuale);
              printf("\n\n\tNella stanza sono presenti degli astronauti. Vuoi fare un'imboscata? Inserisci s/S oppure n/N.\n\t> ");
              scelta = getchar();
              clear_buffer();
              if(scelta != 's' && scelta != 'S' && scelta != 'n' && scelta != 'N'){
                printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: s-S-n-N.\n" white_color);
                sleep(2);
              }
            }while(scelta != 's' && scelta != 'S' && scelta != 'n' && scelta != 'N');
            //Richiamo della funzione uccidi_astronauta() per effettuare l'imboscata
            if(scelta == 's' || scelta == 'S')
                uccidi_astronauta(attuale);
          }
        }else{
          printf("\t" green_color "Non ci sono altre stanze con una botola!\n" white_color);
          sleep(2);
        }
      }else{
        printf("\t" green_color "Questa stanza non ha una botola!\n" white_color);
        sleep(2);
      }
    }else{
      printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n" white_color);
      sleep(1);
    }
  }else{
    printf("\t" red_color "[ERRORE]: Il giocatore non e' valido.\n" white_color);
    sleep(1);
  }
}

//Funzione per sabotare una quest (impostori)
static void sabotaggio(struct Stanza* corrente){
  if(corrente != NULL){
    if(corrente -> tipo == quest_semplice || corrente -> tipo == quest_complicata){
      corrente -> tipo = vuota;
      token = false;
      printf("\t" green_color "La stanza e' stata sabotata!\n" white_color);
      sleep(2);
    }else{
      printf("\t" green_color "Nella stanza non ci sono quest da sabotare!\n" white_color);
      sleep(2);
    }
  }else{
    printf("\t" red_color "[ERRORE]: La stanza non e' valida.\n" white_color);
    sleep(1);
  }
}


//DEFINIZIONE DELLE FUNZIONI PER IL MAIN DICHIARATE IN "gamelib.h"


//Funzione per la stampa del menu principale del gioco
void menu_principale(void){
  titolo();
  printf("\n\n\t" yellow_color " 1] IMPOSTA LA PARTITA\n\t 2] GIOCA\n\t 3] STORIA\n\t 4] RICONOSCIMENTI\n\t 5] ESCI\n\t  > " blue_color);
}

//Funzione per impostare il gioco
void imposta_gioco(void){
  time_t t;
  srand((unsigned) time(&t));
  int scelta;
  //Array per assegnare randomicamente nomi e stati ai giocatori
  int nomi_random[10];
  int stati_random[10];
  short int numero_impostori;
  struct Giocatore* temp = NULL;
  do{
    menu_impostazioni();
    scanf("%d", &scelta);
    clear_buffer();
    switch(scelta){
      case 1:
        //Richiesta del numero di giocatori
        do{
          titolo();
          printf("\n\n\t" yellow_color " Inserisci il numero di giocatori. Ci possono essere minimo 4 e massimo 10 giocatori.\n\t  > " blue_color);
          scanf("%d", &numero_giocatori);
          clear_buffer();
          if(numero_giocatori < 4){
            printf("\t" red_color " [ERRORE]: Il numero minimo di giocatori e' 4.\n" yellow_color);
            sleep(1);
          }else if(numero_giocatori > 10){
            printf("\t" red_color " [ERRORE]: Il numero massimo di giocatori e' 10.\n" yellow_color);
            sleep(1);
          }
        }while(numero_giocatori < 4 || numero_giocatori > 10);
        dealloca();
        giocatori = (struct Giocatore*) calloc(numero_giocatori, sizeof(struct Giocatore));
        if(giocatori == NULL){
          printf("\t" red_color "Memoria esaurita.\n\t" white_color);
          exit(-1);
        }
        //Creazione della stanza di spawn dei giocatori
        stanza_inizio = (struct Stanza*) malloc(sizeof(struct Stanza));
        if(stanza_inizio == NULL){
          printf("\t" red_color "Memoria esaurita.\n\t" white_color);
          exit(-1);
        }
        stanza_inizio -> nord = NULL;
        stanza_inizio -> sud = NULL;
        stanza_inizio -> est = NULL;
        stanza_inizio -> ovest = NULL;
        stanza_inizio -> successiva = NULL;
        stanza_inizio -> nome_stanza = caffetteria;
        stanza_inizio -> x_pos = 0;
        stanza_inizio -> y_pos = 0;
        stanza_inizio -> emergenza_chiamata = false;
        int tipo_nuova_stanza = random(0, 99);
        if(tipo_nuova_stanza < 25)
          stanza_inizio -> tipo = botola;
        else if(tipo_nuova_stanza < 40)
          stanza_inizio -> tipo = quest_complicata;
        else if(tipo_nuova_stanza < 70)
          stanza_inizio -> tipo = quest_semplice;
        else
          stanza_inizio -> tipo = vuota;
        lista_stanze = stanza_inizio;
        //Randomizzazione dei nomi dei giocatori tramite un array
        for(int i = 0; i < 10; i++)
          nomi_random[i] = i;
        random_senza_ripetizioni(nomi_random, 10);
        //Impostazione del numero degli impostori
        if(numero_giocatori < 6)
          numero_impostori = 1;
        else if(numero_giocatori < 8)
          numero_impostori = random(1, 2);
        else
          numero_impostori = random(2, 3);
        //Randomizzazione dello stato dei giocatori tramite un array
        for(int i = 0; i < 10; i++){
          if(i < numero_impostori)
            stati_random[i] = 1;
          else
            stati_random[i] = 0;
        }
        random_senza_ripetizioni(stati_random, numero_giocatori);
        //Inizializzazione dei giocatori
        temp = giocatori;
        for(int i = 0; i < numero_giocatori; i++){
          temp -> posizione = stanza_inizio;
          temp -> stato = stati_random[i];
          temp -> nome = nomi_random[i];
          temp++;
        }
        printf("\t" yellow_color " I giocatori sono stati creati!\n\t" blue_color " Premi invio per tornare al menu." yellow_color);
        clear_buffer();
        break;
      case 2:
        //Impostazione del numero di quest totali da fare per vincere
        do{
          titolo();
          printf("\n\n\t" yellow_color " Inserisci il numero di quest da completare.\n\t  > " blue_color);
          scanf("%hu", &quest_da_finire);
          clear_buffer();
          if(quest_da_finire < 1){
              printf("\t" red_color " [ERRORE]: Il numero di quest da competare non può essere zero.\n" yellow_color);
              sleep(1);
          }
        }while(quest_da_finire < 1);
        printf("\t" yellow_color " Il numero di QUEST da completare e' stato impostato a " blue_color "%hu" yellow_color ".\n\t" blue_color " Premi invio per tornare al menu." yellow_color, quest_da_finire);
        clear_buffer();
        break;
      case 3:
      //Stampa dell'elenco dei giocatori
        titolo();
        stampa_lista_giocatori();
        printf("\t" blue_color " Premi invio per tornare al menu." yellow_color);
        clear_buffer();
        break;
      case 4:
        titolo();
        inizia_gioco();
        break;
      default:
        printf("\t" red_color " [ERRORE]: Comando non valido. Comandi validi: 1-2-3-4.\n" blue_color);
        sleep(2);
    }
  }while(scelta != 4);
}

//Funzione per iniziare la partita
void gioca(void){
  //Verifica se il gioco è stato impostato
  if(giocatori != NULL && quest_da_finire != 0 && stanza_inizio != NULL){
    //Scelta random di chi inizia per primo
    int posizione = random(0, numero_giocatori - 1);
    struct Giocatore* attuale = &giocatori[posizione];
    int scelta;
    quest_completate = 0;
    system("clear");
    printf("\n\n\t" red_color "CI SONO DEGLI IMPOSTORI TRANNOI !!!\n" white_color);
    sleep(3);
    inGioco = true;
    do{
      //Il giocatore gioca il turno solo se è un astronauta o un impostore
      if(attuale -> stato == astronauta || attuale -> stato == impostore){
        system("clear");
        printf("\n\n\t" white_color "TURNO DI: ");
        stampa_nome_giocatore(attuale -> nome);
        printf("\n\tPremi invio per giocare. Assicurati di non essere visto!");
        clear_buffer();
        token = true;
        if(attuale -> stato == astronauta){
          //Scelta mossa per gli astronauti
          do{
            info_turno(attuale);
            printf("\n\n\t" blue_color "MOSSE DA ASTRONAUTA:" white_color "\n\t1] AVANZA IN UN'ALTRA STANZA\n\t2] ESEGUI UNA QUEST\n\t3] PREMI IL PULSANTE DI EMERGENZA\n\t4] SALTA IL TURNO\n\t5] ESCI\n\t > ");
            scanf("%d", &scelta);
            clear_buffer();
            switch(scelta){
              case 1:
                avanza(attuale);
                break;
              case 2:
                esegui_quest(attuale -> posizione);
                break;
              case 3:
                chiamata_emergenza(attuale -> posizione);
                break;
              case 4:
                token = false;
                break;
              case 5:
                esci_partita();
                break;
              default:
                printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: 1-2-3-4-5.\n" white_color);
                sleep(2);
            }
          }while(token);
        }else{
          //Scelta mossa per gli impostori
          do{
            info_turno(attuale);
            printf("\n\n\t" red_color "MOSSE DA IMPOSTORE:" white_color "\n\t1] AVANZA IN UN'ALTRA STANZA\n\t2] UCCIDI UN ASTRONAUTA\n\t3] PREMI IL PULSANTE DI EMERGENZA\n\t4] ENTRA NELLA BOTOLA\n\t5] SABOTA LA QUEST\n\t6] SALTA IL TURNO\n\t7] ESCI\n\t > ");
            scanf("%d", &scelta);
            clear_buffer();
            switch(scelta){
              case 1:
                avanza(attuale);
                break;
              case 2:
                uccidi_astronauta(attuale);
                break;
              case 3:
                chiamata_emergenza(attuale -> posizione);
                break;
              case 4:
                usa_botola(attuale);
                break;
              case 5:
                sabotaggio(attuale -> posizione);
                break;
              case 6:
                token = false;
                break;
              case 7:
                esci_partita();
                break;
              default:
                printf("\t" red_color "[ERRORE]: Comando non valido. Comandi validi: 1-2-3-4-5-6-7.\n" white_color);
                sleep(2);
            }
          }while(token);
        }
      }
      //Dopo il turno si passa al giocatore successivo
      posizione = (posizione + 1) % numero_giocatori;
      attuale = &giocatori[posizione];
      controllo_vittoria();
    }while(inGioco);
    dealloca();
  }else{
    printf("\t" red_color " [ERRORE]: Gioco non impostato. Imposta il gioco dal menu principale prima di avviare una partita.\n" white_color);
    sleep(2);
  }
}

//Funzione per uscire dal gioco
void termina_gioco(void){
  dealloca();
  titolo();
  printf("\n\n\t" green_color " Grazie per aver giocato!\n" white_color);
  sleep(3);
  system("clear");
}

//Funzione per stampare la storia del gioco
void storia(void){
  titolo();
  printf("\n\n\t" yellow_color " STORIA\n\t");
  printf(" Un gruppo di astronauti si trova in viaggio sull’astronave Skelt, verso una lontana galassia,\n\t");
  printf(" in cerca di nuovi pianeti dove vivere e prosperare. Tra di loro di trovano scienziati, astronomi,\n\t");
  printf(" ingegneri e numerosi programmatori da poco laureati all’Università degli Studi di Perugia, nella\n\t");
  printf(" gestione dei computer di bordo. Durante il loro viaggio un grosso meteorite colpisce la nave,\n\t");
  printf(" causando ingenti danni allo scafo e ai sistemi. Tutti si sono messi ad effettuare le relative\n\t");
  printf(" riparazioni per il mantenimento della nave, per poter così arrivare a destinazione sani e salvi.\n\t");
  printf(" Tuttavia tra di loro si celano anche degli impostori, che hanno hackerato i computer di bordo,\n\t");
  printf(" rendendoli non operativi a causa di errori di segmentazione. Questi stanno anche eliminando di\n\t");
  printf(" nascosto gli astronauti senza farsi scoprire da essi, con l’intenzione di prendere il pieno\n\t");
  printf(" controllo della nave. Riusciranno i nostri giovani programmatori ad arrivare a destinazione prima\n\t di essere decimati dai traditori?");
  printf("\n\n\t" blue_color " Premi invio per tornare al menu." yellow_color);
  clear_buffer();
}

//Funzione per stampare i crediti
void crediti(void){
  titolo();
  printf("\n\n\t" yellow_color " TRANNOI! - Crediti di gioco\n");
  printf("\n\t GAME DIRECTOR: Carlicchi Francesco");
  printf("\n\t GAME DESIGNER: Carlicchi Francesco");
  printf("\n\t LEAD DESIGNER: Carlicchi Francesco");
  printf("\n\t PROGRAMMER: Carlicchi Francesco");
  printf("\n\t GAME TESTER: Carlicchi Francesco");
  printf("\n\t @copyright 2021 - tutti i diritti riservati!" blue_color);
  printf("\n\n\t Premi invio per tornare al menu." yellow_color);
  clear_buffer();
}
